@@ -1,5 +1,8 @@
 use super::options::CompilationOptions;
-use crate::schemas;
+use crate::{
+    paths::{InstancePath, JSONPointer, PathChunk},
+    schemas,
+};
 use serde_json::Value;
 use std::borrow::Cow;
 use url::{ParseError, Url};
@@ -8,14 +11,16 @@ use url::{ParseError, Url};
 #[derive(Debug)]
 pub(crate) struct CompilationContext<'a> {
     pub(crate) scope: Cow<'a, Url>,
-    pub(crate) config: Cow<'a, CompilationOptions>,
+    pub(crate) config: &'a CompilationOptions,
+    pub(crate) schema_path: InstancePath<'a>,
 }
 
 impl<'a> CompilationContext<'a> {
-    pub(crate) const fn new(scope: Url, config: Cow<'a, CompilationOptions>) -> Self {
+    pub(crate) const fn new(scope: Url, config: &'a CompilationOptions) -> Self {
         CompilationContext {
             scope: Cow::Owned(scope),
             config,
+            schema_path: InstancePath::new(),
         }
     }
 
@@ -35,16 +40,40 @@ impl<'a> CompilationContext<'a> {
             let scope = Url::options().base_url(Some(&self.scope)).parse(id)?;
             Ok(CompilationContext {
                 scope: Cow::Owned(scope),
-                config: Cow::Borrowed(&self.config),
+                config: self.config,
+                schema_path: self.schema_path.clone(),
             })
         } else {
             Ok(CompilationContext {
                 scope: Cow::Borrowed(self.scope.as_ref()),
-                config: Cow::Borrowed(&self.config),
+                config: self.config,
+                schema_path: self.schema_path.clone(),
             })
         }
     }
 
+    #[inline]
+    pub(crate) fn with_path(&'a self, chunk: impl Into<PathChunk>) -> Self {
+        let schema_path = self.schema_path.push(chunk);
+        CompilationContext {
+            scope: Cow::Borrowed(self.scope.as_ref()),
+            config: self.config,
+            schema_path,
+        }
+    }
+
+    /// Create a JSON Pointer from the current `schema_path` & a new chunk.
+    #[inline]
+    pub(crate) fn into_pointer(self) -> JSONPointer {
+        self.schema_path.into()
+    }
+
+    /// Create a JSON Pointer from the current `schema_path` & a new chunk.
+    #[inline]
+    pub(crate) fn as_pointer_with(&self, chunk: impl Into<PathChunk>) -> JSONPointer {
+        self.schema_path.push(chunk).into()
+    }
+
     /// Build a new URL. Used for `ref` compilation to keep their full paths.
     pub(crate) fn build_url(&self, reference: &str) -> Result<Url, ParseError> {
         Url::options().base_url(Some(&self.scope)).parse(reference)
