@@ -9,34 +9,34 @@ use crate::{
     schemas, ValidationError,
 };
 use ahash::AHashMap;
+use std::{fmt, sync::Arc};
 
-use std::{borrow::Cow, fmt};
+const EXPECT_MESSAGE: &str = "Valid meta-schema!";
 
 lazy_static::lazy_static! {
     static ref DRAFT4:serde_json::Value = serde_json::from_str(include_str!("../../meta_schemas/draft4.json")).expect("Valid schema!");
     static ref DRAFT6:serde_json::Value = serde_json::from_str(include_str!("../../meta_schemas/draft6.json")).expect("Valid schema!");
     static ref DRAFT7:serde_json::Value = serde_json::from_str(include_str!("../../meta_schemas/draft7.json")).expect("Valid schema!");
 
-    static ref META_SCHEMAS: AHashMap<String, serde_json::Value> = {
+    static ref META_SCHEMAS: AHashMap<String, Arc<serde_json::Value>> = {
         let mut store = AHashMap::with_capacity(3);
         store.insert(
             "http://json-schema.org/draft-04/schema".to_string(),
-            DRAFT4.clone()
+            Arc::new(DRAFT4.clone())
         );
         store.insert(
             "http://json-schema.org/draft-06/schema".to_string(),
-            DRAFT6.clone()
+            Arc::new(DRAFT6.clone())
         );
         store.insert(
             "http://json-schema.org/draft-07/schema".to_string(),
-            DRAFT7.clone()
+            Arc::new(DRAFT7.clone())
         );
         store
     };
 
-    static ref META_SCHEMA_VALIDATORS: AHashMap<schemas::Draft, JSONSchema<'static>> = {
+    static ref META_SCHEMA_VALIDATORS: AHashMap<schemas::Draft, JSONSchema> = {
         let mut store = AHashMap::with_capacity(3);
-        const EXPECT_MESSAGE: &str = "Valid meta-schema!";
         store.insert(
             schemas::Draft::Draft4,
             JSONSchema::options().without_schema_validation().compile(&DRAFT4).expect(EXPECT_MESSAGE)
@@ -63,7 +63,8 @@ pub struct CompilationOptions {
     content_media_type_checks: AHashMap<&'static str, Option<ContentMediaTypeCheckType>>,
     content_encoding_checks_and_converters:
         AHashMap<&'static str, Option<(ContentEncodingCheckType, ContentEncodingConverterType)>>,
-    store: AHashMap<String, serde_json::Value>,
+    store: AHashMap<String, Arc<serde_json::Value>>,
+    formats: AHashMap<&'static str, fn(&str) -> bool>,
     validate_schema: bool,
 }
 
@@ -71,10 +72,11 @@ impl Default for CompilationOptions {
     fn default() -> Self {
         CompilationOptions {
             validate_schema: true,
-            draft: Default::default(),
-            content_media_type_checks: Default::default(),
-            content_encoding_checks_and_converters: Default::default(),
-            store: Default::default(),
+            draft: Option::default(),
+            content_media_type_checks: AHashMap::default(),
+            content_encoding_checks_and_converters: AHashMap::default(),
+            store: AHashMap::default(),
+            formats: AHashMap::default(),
         }
     }
 }
@@ -88,7 +90,7 @@ impl CompilationOptions {
     pub fn compile<'a>(
         &self,
         schema: &'a serde_json::Value,
-    ) -> Result<JSONSchema<'a>, ValidationError<'a>> {
+    ) -> Result<JSONSchema, ValidationError<'a>> {
         // Draft is detected in the following precedence order:
         //   - Explicitly specified;
         //   - $schema field in the document;
@@ -106,15 +108,15 @@ impl CompilationOptions {
                 config.with_draft(draft);
             }
         }
-        let processed_config: Cow<'_, CompilationOptions> = Cow::Owned(config);
-        let draft = processed_config.draft();
+        let draft = config.draft();
 
         let scope = match schemas::id_of(draft, schema) {
             Some(url) => url::Url::parse(url)?,
             None => DEFAULT_SCOPE.clone(),
         };
-        let resolver = Resolver::new(draft, &scope, schema, self.store.clone())?;
-        let context = CompilationContext::new(scope, processed_config);
+        let schema_json = Arc::new(schema.clone());
+        let resolver = Resolver::new(draft, &scope, schema_json.clone(), self.store.clone())?;
+        let context = CompilationContext::new(scope, &config);
 
         if self.validate_schema {
             if let Some(mut errors) = META_SCHEMA_VALIDATORS
@@ -131,10 +133,10 @@ impl CompilationOptions {
         validators.shrink_to_fit();
 
         Ok(JSONSchema {
-            schema,
+            schema: schema_json,
             validators,
             resolver,
-            context,
+            config,
         })
     }
 
@@ -344,10 +346,41 @@ impl CompilationOptions {
     /// calls to remote schemas via the `$ref` keyword.
     #[inline]
     pub fn with_document(&mut self, id: String, document: serde_json::Value) -> &mut Self {
-        self.store.insert(id, document);
+        self.store.insert(id, Arc::new(document));
         self
     }
-
+    /// Register a custom "format" validator.
+    ///
+    /// ## Example
+    ///
+    /// ```rust
+    /// # use jsonschema::JSONSchema;
+    /// # use serde_json::json;
+    /// fn my_format(s: &str) -> bool {
+    ///    // Your awesome format check!
+    ///    s.ends_with("42!")
+    /// }
+    /// # fn foo() {
+    /// let schema = json!({"type": "string", "format": "custom"});
+    /// let compiled = JSONSchema::options()
+    ///     .with_format("custom", my_format)
+    ///     .compile(&schema)
+    ///     .expect("Valid schema");
+    /// // Invalid string
+    /// assert!(!compiled.is_valid(&json!("foo")));
+    /// // Valid string
+    /// assert!(compiled.is_valid(&json!("foo42!")));
+    /// # }
+    /// ```
+    ///
+    /// The format check function should receive `&str` and return `bool`.
+    pub fn with_format(&mut self, name: &'static str, format: fn(&str) -> bool) -> &mut Self {
+        self.formats.insert(name, format);
+        self
+    }
+    pub(crate) fn format(&self, format: &str) -> FormatKV<'_> {
+        self.formats.get_key_value(format)
+    }
     /// Do not perform schema validation during compilation.
     /// This method is only used to disable meta-schema validation for meta-schemas itself to avoid
     /// infinite recursion.
@@ -359,6 +392,8 @@ impl CompilationOptions {
         self
     }
 }
+// format name & a pointer to a check function
+type FormatKV<'a> = Option<(&'a &'static str, &'a fn(&str) -> bool)>;
 
 impl fmt::Debug for CompilationOptions {
     fn fmt(&self, fmt: &mut fmt::Formatter<'_>) -> fmt::Result {
@@ -392,7 +427,7 @@ mod tests {
             options.with_draft(draft_version);
         }
         let compiled = options.compile(schema).unwrap();
-        compiled.context.config.draft()
+        compiled.draft()
     }
 
     #[test]
@@ -404,8 +439,23 @@ mod tests {
                 json!({"rule": {"minLength": 5}}),
             )
             .compile(&schema)
-            .unwrap();
+            .expect("Valid schema");
         assert!(!compiled.is_valid(&json!("foo")));
         assert!(compiled.is_valid(&json!("foobar")));
     }
+
+    fn custom(s: &str) -> bool {
+        s.ends_with("42!")
+    }
+
+    #[test]
+    fn custom_format() {
+        let schema = json!({"type": "string", "format": "custom"});
+        let compiled = JSONSchema::options()
+            .with_format("custom", custom)
+            .compile(&schema)
+            .expect("Valid schema");
+        assert!(!compiled.is_valid(&json!("foo")));
+        assert!(compiled.is_valid(&json!("foo42!")));
+    }
 }
