@@ -1,9 +1,8 @@
 use crate::{
     compilation::{compile_validators, context::CompilationContext, JSONSchema},
     error::{ErrorIterator, ValidationError},
-    keywords::{format_validators, format_vec_of_validators, Validators},
     paths::InstancePath,
-    validator::Validate,
+    validator::{format_validators, format_vec_of_validators, Validate, Validators},
 };
 use serde_json::{Map, Value};
 
@@ -19,9 +18,11 @@ impl AllOfValidator {
         items: &'a [Value],
         context: &CompilationContext,
     ) -> CompilationResult<'a> {
+        let keyword_context = context.with_path("allOf");
         let mut schemas = Vec::with_capacity(items.len());
-        for item in items {
-            let validators = compile_validators(item, context)?;
+        for (idx, item) in items.iter().enumerate() {
+            let item_context = keyword_context.with_path(idx);
+            let validators = compile_validators(item, &item_context)?;
             schemas.push(validators)
         }
         Ok(Box::new(AllOfValidator { schemas }))
@@ -37,12 +38,12 @@ impl Validate for AllOfValidator {
         })
     }
 
-    fn validate<'a>(
+    fn validate<'a, 'b>(
         &self,
         schema: &'a JSONSchema,
-        instance: &'a Value,
+        instance: &'b Value,
         instance_path: &InstancePath,
-    ) -> ErrorIterator<'a> {
+    ) -> ErrorIterator<'b> {
         let errors: Vec<_> = self
             .schemas
             .iter()
@@ -56,9 +57,9 @@ impl Validate for AllOfValidator {
     }
 }
 
-impl ToString for AllOfValidator {
-    fn to_string(&self) -> String {
-        format!("allOf: [{}]", format_vec_of_validators(&self.schemas))
+impl core::fmt::Display for AllOfValidator {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(f, "allOf: [{}]", format_vec_of_validators(&self.schemas))
     }
 }
 pub(crate) struct SingleValueAllOfValidator {
@@ -71,7 +72,9 @@ impl SingleValueAllOfValidator {
         schema: &'a Value,
         context: &CompilationContext,
     ) -> CompilationResult<'a> {
-        let validators = compile_validators(schema, context)?;
+        let keyword_context = context.with_path("allOf");
+        let item_context = keyword_context.with_path(0);
+        let validators = compile_validators(schema, &item_context)?;
         Ok(Box::new(SingleValueAllOfValidator { validators }))
     }
 }
@@ -83,12 +86,12 @@ impl Validate for SingleValueAllOfValidator {
             .all(move |validator| validator.is_valid(schema, instance))
     }
 
-    fn validate<'a>(
+    fn validate<'a, 'b>(
         &self,
         schema: &'a JSONSchema,
-        instance: &'a Value,
+        instance: &'b Value,
         instance_path: &InstancePath,
-    ) -> ErrorIterator<'a> {
+    ) -> ErrorIterator<'b> {
         let errors: Vec<_> = self
             .validators
             .iter()
@@ -98,9 +101,9 @@ impl Validate for SingleValueAllOfValidator {
     }
 }
 
-impl ToString for SingleValueAllOfValidator {
-    fn to_string(&self) -> String {
-        format!("allOf: [{}]", format_validators(&self.validators))
+impl core::fmt::Display for SingleValueAllOfValidator {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(f, "allOf: [{}]", format_validators(&self.validators))
     }
 }
 #[inline]
@@ -120,3 +123,16 @@ pub(crate) fn compile<'a>(
         Some(Err(ValidationError::schema(schema)))
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use crate::tests_util;
+    use serde_json::{json, Value};
+    use test_case::test_case;
+
+    #[test_case(&json!({"allOf": [{"type": "string"}]}), &json!(1), "/allOf/0/type")]
+    #[test_case(&json!({"allOf": [{"type": "integer"}, {"maximum": 5}]}), &json!(6), "/allOf/1/maximum")]
+    fn schema_path(schema: &Value, instance: &Value, expected: &str) {
+        tests_util::assert_schema_path(schema, instance, expected)
+    }
+}
