@@ -1,14 +1,15 @@
 use crate::{
     compilation::{compile_validators, context::CompilationContext, JSONSchema},
     error::{error, no_error, ErrorIterator, ValidationError},
-    keywords::{format_validators, CompilationResult, Validators},
-    paths::InstancePath,
-    validator::Validate,
+    keywords::CompilationResult,
+    paths::{InstancePath, JSONPointer},
+    validator::{format_validators, Validate, Validators},
 };
 use serde_json::{Map, Value};
 
 pub(crate) struct ContainsValidator {
     validators: Validators,
+    schema_path: JSONPointer,
 }
 
 impl ContainsValidator {
@@ -17,8 +18,10 @@ impl ContainsValidator {
         schema: &'a Value,
         context: &CompilationContext,
     ) -> CompilationResult<'a> {
+        let keyword_context = context.with_path("contains");
         Ok(Box::new(ContainsValidator {
-            validators: compile_validators(schema, context)?,
+            validators: compile_validators(schema, &keyword_context)?,
+            schema_path: keyword_context.into_pointer(),
         }))
     }
 }
@@ -41,12 +44,12 @@ impl Validate for ContainsValidator {
         }
     }
 
-    fn validate<'a>(
+    fn validate<'a, 'b>(
         &self,
         schema: &'a JSONSchema,
-        instance: &'a Value,
+        instance: &'b Value,
         instance_path: &InstancePath,
-    ) -> ErrorIterator<'a> {
+    ) -> ErrorIterator<'b> {
         if let Value::Array(items) = instance {
             for item in items {
                 if self
@@ -57,16 +60,20 @@ impl Validate for ContainsValidator {
                     return no_error();
                 }
             }
-            error(ValidationError::contains(instance_path.into(), instance))
+            error(ValidationError::contains(
+                self.schema_path.clone(),
+                instance_path.into(),
+                instance,
+            ))
         } else {
             no_error()
         }
     }
 }
 
-impl ToString for ContainsValidator {
-    fn to_string(&self) -> String {
-        format!("contains: {}", format_validators(&self.validators))
+impl core::fmt::Display for ContainsValidator {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(f, "contains: {}", format_validators(&self.validators))
     }
 }
 
@@ -78,3 +85,14 @@ pub(crate) fn compile<'a>(
 ) -> Option<CompilationResult<'a>> {
     Some(ContainsValidator::compile(schema, context))
 }
+
+#[cfg(test)]
+mod tests {
+    use crate::tests_util;
+    use serde_json::json;
+
+    #[test]
+    fn schema_path() {
+        tests_util::assert_schema_path(&json!({"contains": {"const": 2}}), &json!([]), "/contains")
+    }
+}
