@@ -1,9 +1,9 @@
 use crate::{
     compilation::{compile_validators, context::CompilationContext, JSONSchema},
     error::{no_error, ErrorIterator, ValidationError},
-    keywords::{format_key_value_validators, required, CompilationResult, Validators},
+    keywords::{required, CompilationResult},
     paths::InstancePath,
-    validator::Validate,
+    validator::{format_key_value_validators, Validate, Validators},
 };
 use serde_json::{Map, Value};
 
@@ -18,14 +18,19 @@ impl DependenciesValidator {
         context: &CompilationContext,
     ) -> CompilationResult<'a> {
         if let Value::Object(map) = schema {
+            let keyword_context = context.with_path("dependencies");
             let mut dependencies = Vec::with_capacity(map.len());
             for (key, subschema) in map {
+                let item_context = keyword_context.with_path(key.to_string());
                 let s = match subschema {
                     Value::Array(_) => {
-                        vec![required::compile(map, subschema, context)
-                            .expect("The required validator compilation does not return None")?]
+                        vec![required::compile_with_path(
+                            subschema,
+                            (&keyword_context.schema_path).into(),
+                        )
+                        .expect("The required validator compilation does not return None")?]
                     }
-                    _ => compile_validators(subschema, context)?,
+                    _ => compile_validators(subschema, &item_context)?,
                 };
                 dependencies.push((key.clone(), s))
             }
@@ -52,12 +57,12 @@ impl Validate for DependenciesValidator {
         }
     }
 
-    fn validate<'a>(
+    fn validate<'a, 'b>(
         &self,
         schema: &'a JSONSchema,
-        instance: &'a Value,
+        instance: &'b Value,
         instance_path: &InstancePath,
-    ) -> ErrorIterator<'a> {
+    ) -> ErrorIterator<'b> {
         if let Value::Object(item) = instance {
             let errors: Vec<_> = self
                 .dependencies
@@ -77,9 +82,10 @@ impl Validate for DependenciesValidator {
     }
 }
 
-impl ToString for DependenciesValidator {
-    fn to_string(&self) -> String {
-        format!(
+impl core::fmt::Display for DependenciesValidator {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(
+            f,
             "dependencies: {{{}}}",
             format_key_value_validators(&self.dependencies)
         )
@@ -94,3 +100,16 @@ pub(crate) fn compile<'a>(
 ) -> Option<CompilationResult<'a>> {
     Some(DependenciesValidator::compile(schema, context))
 }
+
+#[cfg(test)]
+mod tests {
+    use crate::tests_util;
+    use serde_json::{json, Value};
+    use test_case::test_case;
+
+    #[test_case(&json!({"dependencies": {"bar": ["foo"]}}), &json!({"bar": 1}), "/dependencies")]
+    #[test_case(&json!({"dependencies": {"bar": {"type": "string"}}}), &json!({"bar": 1}), "/dependencies/bar/type")]
+    fn schema_path(schema: &Value, instance: &Value, expected: &str) {
+        tests_util::assert_schema_path(schema, instance, expected)
+    }
+}
