@@ -1,9 +1,9 @@
 use crate::{
     compilation::{compile_validators, context::CompilationContext, JSONSchema},
     error::{no_error, ErrorIterator},
-    keywords::{format_validators, format_vec_of_validators, CompilationResult, Validators},
+    keywords::CompilationResult,
     paths::InstancePath,
-    validator::Validate,
+    validator::{format_validators, format_vec_of_validators, Validate, Validators},
 };
 use serde_json::{Map, Value};
 
@@ -16,9 +16,11 @@ impl ItemsArrayValidator {
         schemas: &'a [Value],
         context: &CompilationContext,
     ) -> CompilationResult<'a> {
+        let keyword_context = context.with_path("items");
         let mut items = Vec::with_capacity(schemas.len());
-        for item in schemas {
-            let validators = compile_validators(item, context)?;
+        for (idx, item) in schemas.iter().enumerate() {
+            let item_context = keyword_context.with_path(idx);
+            let validators = compile_validators(item, &item_context)?;
             items.push(validators)
         }
         Ok(Box::new(ItemsArrayValidator { items }))
@@ -40,12 +42,12 @@ impl Validate for ItemsArrayValidator {
         }
     }
 
-    fn validate<'a>(
+    fn validate<'a, 'b>(
         &self,
         schema: &'a JSONSchema,
-        instance: &'a Value,
+        instance: &'b Value,
         instance_path: &InstancePath,
-    ) -> ErrorIterator<'a> {
+    ) -> ErrorIterator<'b> {
         if let Value::Array(items) = instance {
             let errors: Vec<_> = items
                 .iter()
@@ -64,9 +66,9 @@ impl Validate for ItemsArrayValidator {
     }
 }
 
-impl ToString for ItemsArrayValidator {
-    fn to_string(&self) -> String {
-        format!("items: [{}]", format_vec_of_validators(&self.items))
+impl core::fmt::Display for ItemsArrayValidator {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(f, "items: [{}]", format_vec_of_validators(&self.items))
     }
 }
 
@@ -79,7 +81,8 @@ impl ItemsObjectValidator {
         schema: &'a Value,
         context: &CompilationContext,
     ) -> CompilationResult<'a> {
-        let validators = compile_validators(schema, context)?;
+        let keyword_context = context.with_path("items");
+        let validators = compile_validators(schema, &keyword_context)?;
         Ok(Box::new(ItemsObjectValidator { validators }))
     }
 }
@@ -96,12 +99,12 @@ impl Validate for ItemsObjectValidator {
         }
     }
 
-    fn validate<'a>(
+    fn validate<'a, 'b>(
         &self,
         schema: &'a JSONSchema,
-        instance: &'a Value,
+        instance: &'b Value,
         instance_path: &InstancePath,
-    ) -> ErrorIterator<'a> {
+    ) -> ErrorIterator<'b> {
         if let Value::Array(items) = instance {
             let errors: Vec<_> = self
                 .validators
@@ -119,9 +122,9 @@ impl Validate for ItemsObjectValidator {
     }
 }
 
-impl ToString for ItemsObjectValidator {
-    fn to_string(&self) -> String {
-        format!("items: {}", format_validators(&self.validators))
+impl core::fmt::Display for ItemsObjectValidator {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(f, "items: {}", format_validators(&self.validators))
     }
 }
 
@@ -144,3 +147,17 @@ pub(crate) fn compile<'a>(
         _ => None,
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use crate::tests_util;
+    use serde_json::{json, Value};
+    use test_case::test_case;
+
+    #[test_case(&json!({"items": false}), &json!([1]), "/items")]
+    #[test_case(&json!({"items": {"type": "string"}}), &json!([1]), "/items/type")]
+    #[test_case(&json!({"items": [{"type": "string"}]}), &json!([1]), "/items/0/type")]
+    fn schema_path(schema: &Value, instance: &Value, expected: &str) {
+        tests_util::assert_schema_path(schema, instance, expected)
+    }
+}
