@@ -2,7 +2,7 @@ use crate::{
     compilation::{context::CompilationContext, JSONSchema},
     error::{error, no_error, ErrorIterator, ValidationError},
     keywords::CompilationResult,
-    paths::InstancePath,
+    paths::{InstancePath, JSONPointer},
     validator::Validate,
 };
 use num_cmp::NumCmp;
@@ -10,27 +10,31 @@ use serde_json::{Map, Value};
 
 pub(crate) struct MaximumU64Validator {
     limit: u64,
+    schema_path: JSONPointer,
 }
 pub(crate) struct MaximumI64Validator {
     limit: i64,
+    schema_path: JSONPointer,
 }
 pub(crate) struct MaximumF64Validator {
     limit: f64,
+    schema_path: JSONPointer,
 }
 
 macro_rules! validate {
     ($validator: ty) => {
         impl Validate for $validator {
-            fn validate<'a>(
+            fn validate<'a, 'b>(
                 &self,
                 schema: &'a JSONSchema,
-                instance: &'a Value,
+                instance: &'b Value,
                 instance_path: &InstancePath,
-            ) -> ErrorIterator<'a> {
+            ) -> ErrorIterator<'b> {
                 if self.is_valid(schema, instance) {
                     no_error()
                 } else {
                     error(ValidationError::maximum(
+                        self.schema_path.clone(),
                         instance_path.into(),
                         instance,
                         self.limit as f64,
@@ -52,9 +56,9 @@ macro_rules! validate {
                 true
             }
         }
-        impl ToString for $validator {
-            fn to_string(&self) -> String {
-                format!("maximum: {}", self.limit)
+        impl core::fmt::Display for $validator {
+            fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+                write!(f, "maximum: {}", self.limit)
             }
         }
     };
@@ -78,16 +82,17 @@ impl Validate for MaximumF64Validator {
         true
     }
 
-    fn validate<'a>(
+    fn validate<'a, 'b>(
         &self,
         schema: &'a JSONSchema,
-        instance: &'a Value,
+        instance: &'b Value,
         instance_path: &InstancePath,
-    ) -> ErrorIterator<'a> {
+    ) -> ErrorIterator<'b> {
         if self.is_valid(schema, instance) {
             no_error()
         } else {
             error(ValidationError::maximum(
+                self.schema_path.clone(),
                 instance_path.into(),
                 instance,
                 self.limit,
@@ -95,9 +100,9 @@ impl Validate for MaximumF64Validator {
         }
     }
 }
-impl ToString for MaximumF64Validator {
-    fn to_string(&self) -> String {
-        format!("maximum: {}", self.limit)
+impl core::fmt::Display for MaximumF64Validator {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(f, "maximum: {}", self.limit)
     }
 }
 
@@ -105,16 +110,17 @@ impl ToString for MaximumF64Validator {
 pub(crate) fn compile<'a>(
     _: &'a Map<String, Value>,
     schema: &'a Value,
-    _: &CompilationContext,
+    context: &CompilationContext,
 ) -> Option<CompilationResult<'a>> {
     if let Value::Number(limit) = schema {
+        let schema_path = context.as_pointer_with("maximum");
         if let Some(limit) = limit.as_u64() {
-            Some(Ok(Box::new(MaximumU64Validator { limit })))
+            Some(Ok(Box::new(MaximumU64Validator { limit, schema_path })))
         } else if let Some(limit) = limit.as_i64() {
-            Some(Ok(Box::new(MaximumI64Validator { limit })))
+            Some(Ok(Box::new(MaximumI64Validator { limit, schema_path })))
         } else {
             let limit = limit.as_f64().expect("Always valid");
-            Some(Ok(Box::new(MaximumF64Validator { limit })))
+            Some(Ok(Box::new(MaximumF64Validator { limit, schema_path })))
         }
     } else {
         Some(Err(ValidationError::schema(schema)))
@@ -127,9 +133,16 @@ mod tests {
     use serde_json::{json, Value};
     use test_case::test_case;
 
-    #[test_case(&json!({"maximum": 1u64 << 54}), &json!((1u64 << 54) + 1))]
-    #[test_case(&json!({"maximum": 1i64 << 54}), &json!((1i64 << 54) + 1))]
+    #[test_case(&json!({"maximum": 1_u64 << 54}), &json!((1_u64 << 54) + 1))]
+    #[test_case(&json!({"maximum": 1_i64 << 54}), &json!((1_i64 << 54) + 1))]
     fn is_not_valid(schema: &Value, instance: &Value) {
         tests_util::is_not_valid(schema, instance)
     }
+
+    #[test_case(&json!({"maximum": 5}), &json!(10), "/maximum")]
+    #[test_case(&json!({"maximum": 6}), &json!(10), "/maximum")]
+    #[test_case(&json!({"maximum": 7}), &json!(10), "/maximum")]
+    fn schema_path(schema: &Value, instance: &Value, expected: &str) {
+        tests_util::assert_schema_path(schema, instance, expected)
+    }
 }
