@@ -1,14 +1,15 @@
 use crate::{
     compilation::{compile_validators, context::CompilationContext, JSONSchema},
     error::{error, no_error, ErrorIterator, ValidationError},
-    keywords::{format_vec_of_validators, CompilationResult, Validators},
-    paths::InstancePath,
-    validator::Validate,
+    keywords::CompilationResult,
+    paths::{InstancePath, JSONPointer},
+    validator::{format_vec_of_validators, Validate, Validators},
 };
 use serde_json::{Map, Value};
 
 pub(crate) struct OneOfValidator {
     schemas: Vec<Validators>,
+    schema_path: JSONPointer,
 }
 
 impl OneOfValidator {
@@ -18,11 +19,15 @@ impl OneOfValidator {
         context: &CompilationContext,
     ) -> CompilationResult<'a> {
         if let Value::Array(items) = schema {
+            let keyword_context = context.with_path("oneOf");
             let mut schemas = Vec::with_capacity(items.len());
             for item in items {
-                schemas.push(compile_validators(item, context)?)
+                schemas.push(compile_validators(item, &keyword_context)?)
             }
-            Ok(Box::new(OneOfValidator { schemas }))
+            Ok(Box::new(OneOfValidator {
+                schemas,
+                schema_path: keyword_context.into_pointer(),
+            }))
         } else {
             Err(ValidationError::schema(schema))
         }
@@ -64,16 +69,17 @@ impl Validate for OneOfValidator {
         let first_valid_idx = self.get_first_valid(schema, instance);
         first_valid_idx.map_or(false, |idx| !self.are_others_valid(schema, instance, idx))
     }
-    fn validate<'a>(
+    fn validate<'a, 'b>(
         &self,
         schema: &'a JSONSchema,
-        instance: &'a Value,
+        instance: &'b Value,
         instance_path: &InstancePath,
-    ) -> ErrorIterator<'a> {
+    ) -> ErrorIterator<'b> {
         let first_valid_idx = self.get_first_valid(schema, instance);
         if let Some(idx) = first_valid_idx {
             if self.are_others_valid(schema, instance, idx) {
                 return error(ValidationError::one_of_multiple_valid(
+                    self.schema_path.clone(),
                     instance_path.into(),
                     instance,
                 ));
@@ -81,6 +87,7 @@ impl Validate for OneOfValidator {
             no_error()
         } else {
             error(ValidationError::one_of_not_valid(
+                self.schema_path.clone(),
                 instance_path.into(),
                 instance,
             ))
@@ -88,9 +95,9 @@ impl Validate for OneOfValidator {
     }
 }
 
-impl ToString for OneOfValidator {
-    fn to_string(&self) -> String {
-        format!("oneOf: [{}]", format_vec_of_validators(&self.schemas))
+impl core::fmt::Display for OneOfValidator {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(f, "oneOf: [{}]", format_vec_of_validators(&self.schemas))
     }
 }
 
@@ -102,3 +109,16 @@ pub(crate) fn compile<'a>(
 ) -> Option<CompilationResult<'a>> {
     Some(OneOfValidator::compile(schema, context))
 }
+
+#[cfg(test)]
+mod tests {
+    use crate::tests_util;
+    use serde_json::{json, Value};
+    use test_case::test_case;
+
+    #[test_case(&json!({"oneOf": [{"type": "string"}]}), &json!(0), "/oneOf")]
+    #[test_case(&json!({"oneOf": [{"type": "string"}, {"maxLength": 3}]}), &json!(""), "/oneOf")]
+    fn schema_path(schema: &Value, instance: &Value, expected: &str) {
+        tests_util::assert_schema_path(schema, instance, expected)
+    }
+}
