@@ -1,9 +1,9 @@
 use crate::{
     compilation::{compile_validators, context::CompilationContext, JSONSchema},
     error::{no_error, ErrorIterator, ValidationError},
-    keywords::{format_validators, CompilationResult, Validators},
+    keywords::CompilationResult,
     paths::InstancePath,
-    validator::Validate,
+    validator::{format_validators, Validate, Validators},
 };
 use fancy_regex::Regex;
 use serde_json::{Map, Value};
@@ -18,14 +18,16 @@ impl PatternPropertiesValidator {
         map: &'a Map<String, Value>,
         context: &CompilationContext,
     ) -> CompilationResult<'a> {
+        let keyword_context = context.with_path("patternProperties");
         let mut patterns = Vec::with_capacity(map.len());
         for (pattern, subschema) in map {
+            let pattern_context = keyword_context.with_path(pattern.to_string());
             patterns.push((
                 match Regex::new(pattern) {
                     Ok(r) => r,
                     Err(_) => return Err(ValidationError::schema(subschema)),
                 },
-                compile_validators(subschema, context)?,
+                compile_validators(subschema, &pattern_context)?,
             ));
         }
         Ok(Box::new(PatternPropertiesValidator { patterns }))
@@ -49,12 +51,12 @@ impl Validate for PatternPropertiesValidator {
         }
     }
 
-    fn validate<'a>(
+    fn validate<'a, 'b>(
         &self,
         schema: &'a JSONSchema,
-        instance: &'a Value,
+        instance: &'b Value,
         instance_path: &InstancePath,
-    ) -> ErrorIterator<'a> {
+    ) -> ErrorIterator<'b> {
         if let Value::Object(item) = instance {
             let errors: Vec<_> = self
                 .patterns
@@ -63,7 +65,7 @@ impl Validate for PatternPropertiesValidator {
                     item.iter()
                         .filter(move |(key, _)| re.is_match(key).unwrap_or(false))
                         .flat_map(move |(key, value)| {
-                            let instance_path = instance_path.push(key.to_owned());
+                            let instance_path = instance_path.push(key.clone());
                             validators.iter().flat_map(move |validator| {
                                 validator.validate(schema, value, &instance_path)
                             })
@@ -77,9 +79,10 @@ impl Validate for PatternPropertiesValidator {
     }
 }
 
-impl ToString for PatternPropertiesValidator {
-    fn to_string(&self) -> String {
-        format!(
+impl core::fmt::Display for PatternPropertiesValidator {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(
+            f,
             "patternProperties: {{{}}}",
             self.patterns
                 .iter()
@@ -102,12 +105,14 @@ impl SingleValuePatternPropertiesValidator {
         schema: &'a Value,
         context: &CompilationContext,
     ) -> CompilationResult<'a> {
+        let keyword_context = context.with_path("patternProperties");
+        let pattern_context = keyword_context.with_path(pattern.to_string());
         Ok(Box::new(SingleValuePatternPropertiesValidator {
             pattern: match Regex::new(pattern) {
                 Ok(r) => r,
                 Err(_) => return Err(ValidationError::schema(schema)),
             },
-            validators: compile_validators(schema, context)?,
+            validators: compile_validators(schema, &pattern_context)?,
         }))
     }
 }
@@ -127,18 +132,18 @@ impl Validate for SingleValuePatternPropertiesValidator {
         }
     }
 
-    fn validate<'a>(
+    fn validate<'a, 'b>(
         &self,
         schema: &'a JSONSchema,
-        instance: &'a Value,
+        instance: &'b Value,
         instance_path: &InstancePath,
-    ) -> ErrorIterator<'a> {
+    ) -> ErrorIterator<'b> {
         if let Value::Object(item) = instance {
             let errors: Vec<_> = item
                 .iter()
                 .filter(move |(key, _)| self.pattern.is_match(key).unwrap_or(false))
                 .flat_map(move |(key, value)| {
-                    let instance_path = instance_path.push(key.to_owned());
+                    let instance_path = instance_path.push(key.clone());
                     self.validators.iter().flat_map(move |validator| {
                         validator.validate(schema, value, &instance_path)
                     })
@@ -151,9 +156,10 @@ impl Validate for SingleValuePatternPropertiesValidator {
     }
 }
 
-impl ToString for SingleValuePatternPropertiesValidator {
-    fn to_string(&self) -> String {
-        format!(
+impl core::fmt::Display for SingleValuePatternPropertiesValidator {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(
+            f,
             "patternProperties: {{{}: {}}}",
             self.pattern,
             format_validators(&self.validators)
@@ -186,3 +192,16 @@ pub(crate) fn compile<'a>(
         }
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use crate::tests_util;
+    use serde_json::{json, Value};
+    use test_case::test_case;
+
+    #[test_case(&json!({"patternProperties": {"^f": {"type": "string"}}}), &json!({"f": 42}), "/patternProperties/^f/type")]
+    #[test_case(&json!({"patternProperties": {"^f": {"type": "string"}, "^x": {"type": "string"}}}), &json!({"f": 42}), "/patternProperties/^f/type")]
+    fn schema_path(schema: &Value, instance: &Value, expected: &str) {
+        tests_util::assert_schema_path(schema, instance, expected)
+    }
+}
