@@ -1,12 +1,11 @@
 use crate::{
     compilation::{compile_validators, context::CompilationContext, JSONSchema},
     error::{error, no_error, ErrorIterator, ValidationError},
-    keywords::{format_validators, CompilationResult, Validators},
-    paths::InstancePath,
-    validator::Validate,
+    keywords::CompilationResult,
+    paths::{InstancePath, JSONPointer},
+    validator::{format_validators, Validate, Validators},
 };
 use serde_json::{Map, Value};
-use std::borrow::Borrow;
 
 pub(crate) struct PropertyNamesObjectValidator {
     validators: Validators,
@@ -18,16 +17,17 @@ impl PropertyNamesObjectValidator {
         schema: &'a Value,
         context: &CompilationContext,
     ) -> CompilationResult<'a> {
+        let keyword_context = context.with_path("propertyNames");
         Ok(Box::new(PropertyNamesObjectValidator {
-            validators: compile_validators(schema, context)?,
+            validators: compile_validators(schema, &keyword_context)?,
         }))
     }
 }
 
 impl Validate for PropertyNamesObjectValidator {
     fn is_valid(&self, schema: &JSONSchema, instance: &Value) -> bool {
-        if let Value::Object(item) = &instance.borrow() {
-            self.validators.iter().all(move |validator| {
+        if let Value::Object(item) = &instance {
+            self.validators.iter().all(|validator| {
                 item.keys().all(move |key| {
                     let wrapper = Value::String(key.to_string());
                     validator.is_valid(schema, &wrapper)
@@ -38,23 +38,24 @@ impl Validate for PropertyNamesObjectValidator {
         }
     }
 
-    fn validate<'a>(
+    fn validate<'a, 'b>(
         &self,
         schema: &'a JSONSchema,
-        instance: &'a Value,
+        instance: &'b Value,
         instance_path: &InstancePath,
-    ) -> ErrorIterator<'a> {
-        if let Value::Object(item) = &instance.borrow() {
+    ) -> ErrorIterator<'b> {
+        if let Value::Object(item) = &instance {
             let errors: Vec<_> = self
                 .validators
                 .iter()
-                .flat_map(move |validator| {
+                .flat_map(|validator| {
                     item.keys().flat_map(move |key| {
                         let wrapper = Value::String(key.to_string());
                         let errors: Vec<_> = validator
                             .validate(schema, &wrapper, instance_path)
                             .map(|error| {
                                 ValidationError::property_names(
+                                    error.schema_path.clone(),
                                     instance_path.into(),
                                     instance,
                                     error.into_owned(),
@@ -71,18 +72,21 @@ impl Validate for PropertyNamesObjectValidator {
         }
     }
 }
-impl ToString for PropertyNamesObjectValidator {
-    fn to_string(&self) -> String {
-        format!("propertyNames: {}", format_validators(&self.validators))
+impl core::fmt::Display for PropertyNamesObjectValidator {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(f, "propertyNames: {}", format_validators(&self.validators))
     }
 }
 
-pub(crate) struct PropertyNamesBooleanValidator {}
+pub(crate) struct PropertyNamesBooleanValidator {
+    schema_path: JSONPointer,
+}
 
 impl PropertyNamesBooleanValidator {
     #[inline]
-    pub(crate) fn compile<'a>() -> CompilationResult<'a> {
-        Ok(Box::new(PropertyNamesBooleanValidator {}))
+    pub(crate) fn compile<'a>(context: &CompilationContext) -> CompilationResult<'a> {
+        let schema_path = context.as_pointer_with("propertyNames");
+        Ok(Box::new(PropertyNamesBooleanValidator { schema_path }))
     }
 }
 
@@ -96,16 +100,17 @@ impl Validate for PropertyNamesBooleanValidator {
         true
     }
 
-    fn validate<'a>(
+    fn validate<'a, 'b>(
         &self,
         schema: &'a JSONSchema,
-        instance: &'a Value,
+        instance: &'b Value,
         instance_path: &InstancePath,
-    ) -> ErrorIterator<'a> {
+    ) -> ErrorIterator<'b> {
         if self.is_valid(schema, instance) {
             no_error()
         } else {
             error(ValidationError::false_schema(
+                self.schema_path.clone(),
                 instance_path.into(),
                 instance,
             ))
@@ -113,9 +118,9 @@ impl Validate for PropertyNamesBooleanValidator {
     }
 }
 
-impl ToString for PropertyNamesBooleanValidator {
-    fn to_string(&self) -> String {
-        "propertyNames: false".to_string()
+impl core::fmt::Display for PropertyNamesBooleanValidator {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        "propertyNames: false".fmt(f)
     }
 }
 
@@ -127,7 +132,20 @@ pub(crate) fn compile<'a>(
 ) -> Option<CompilationResult<'a>> {
     match schema {
         Value::Object(_) => Some(PropertyNamesObjectValidator::compile(schema, context)),
-        Value::Bool(false) => Some(PropertyNamesBooleanValidator::compile()),
+        Value::Bool(false) => Some(PropertyNamesBooleanValidator::compile(context)),
         _ => None,
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use crate::tests_util;
+    use serde_json::{json, Value};
+    use test_case::test_case;
+
+    #[test_case(&json!({"propertyNames": false}), &json!({"foo": 1}), "/propertyNames")]
+    #[test_case(&json!({"propertyNames": {"minLength": 2}}), &json!({"f": 1}), "/propertyNames/minLength")]
+    fn schema_path(schema: &Value, instance: &Value, expected: &str) {
+        tests_util::assert_schema_path(schema, instance, expected)
+    }
+}
