@@ -1,13 +1,12 @@
 use crate::{
     compilation::{compile_validators, context::CompilationContext, JSONSchema},
     error::{error, ErrorIterator},
-    keywords::{CompilationResult, Validators},
-    paths::InstancePath,
-    validator::Validate,
+    keywords::CompilationResult,
+    paths::{InstancePath, JSONPointer},
+    validator::{Validate, Validators},
 };
 use parking_lot::RwLock;
 use serde_json::Value;
-use std::borrow::Cow;
 use url::Url;
 
 pub(crate) struct RefValidator {
@@ -18,6 +17,7 @@ pub(crate) struct RefValidator {
     /// and at the same time during validation we iterate over shared
     /// references (&self) and not owned references (&mut self).
     validators: RwLock<Option<Validators>>,
+    schema_path: JSONPointer,
 }
 
 impl RefValidator {
@@ -30,6 +30,7 @@ impl RefValidator {
         Ok(Box::new(RefValidator {
             reference,
             validators: RwLock::new(None),
+            schema_path: context.schema_path.clone().into(),
         }))
     }
 }
@@ -41,12 +42,11 @@ impl Validate for RefValidator {
                 .iter()
                 .all(move |validator| validator.is_valid(schema, instance));
         }
-        if let Ok((scope, resolved)) = schema.resolver.resolve_fragment(
-            schema.context.config.draft(),
-            &self.reference,
-            schema.schema,
-        ) {
-            let context = CompilationContext::new(scope, Cow::Borrowed(&schema.context.config));
+        if let Ok((scope, resolved)) = schema
+            .resolver
+            .resolve_fragment(schema.draft(), &self.reference)
+        {
+            let context = CompilationContext::new(scope, schema.config());
             if let Ok(validators) = compile_validators(&resolved, &context) {
                 let result = validators
                     .iter()
@@ -58,12 +58,12 @@ impl Validate for RefValidator {
         false
     }
 
-    fn validate<'a>(
+    fn validate<'a, 'b>(
         &self,
         schema: &'a JSONSchema,
-        instance: &'a Value,
+        instance: &'b Value,
         instance_path: &InstancePath,
-    ) -> ErrorIterator<'a> {
+    ) -> ErrorIterator<'b> {
         if let Some(validators) = self.validators.read().as_ref() {
             return Box::new(
                 validators
@@ -73,20 +73,27 @@ impl Validate for RefValidator {
                     .into_iter(),
             );
         }
-        match schema.resolver.resolve_fragment(
-            schema.context.config.draft(),
-            &self.reference,
-            schema.schema,
-        ) {
+        match schema
+            .resolver
+            .resolve_fragment(schema.draft(), &self.reference)
+        {
             Ok((scope, resolved)) => {
-                let context = CompilationContext::new(scope, Cow::Borrowed(&schema.context.config));
+                let context = CompilationContext::new(scope, schema.config());
                 match compile_validators(&resolved, &context) {
                     Ok(validators) => {
                         let result = Box::new(
                             validators
                                 .iter()
                                 .flat_map(move |validator| {
-                                    validator.validate(schema, instance, instance_path)
+                                    let schema_path = self.schema_path.clone();
+                                    validator.validate(schema, instance, instance_path).map(
+                                        move |mut error| {
+                                            // Prepend $ref path to the actual error
+                                            error.schema_path = schema_path
+                                                .extend_with(error.schema_path.as_slice());
+                                            error
+                                        },
+                                    )
                                 })
                                 .collect::<Vec<_>>()
                                 .into_iter(),
@@ -97,14 +104,14 @@ impl Validate for RefValidator {
                     Err(err) => error(err.into_owned()),
                 }
             }
-            Err(err) => error(err),
+            Err(err) => error(err.into_owned()),
         }
     }
 }
 
-impl ToString for RefValidator {
-    fn to_string(&self) -> String {
-        format!("$ref: {}", self.reference)
+impl core::fmt::Display for RefValidator {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(f, "$ref: {}", self.reference)
     }
 }
 
@@ -116,3 +123,18 @@ pub(crate) fn compile<'a>(
 ) -> Option<CompilationResult<'a>> {
     Some(RefValidator::compile(reference, context))
 }
+
+#[cfg(test)]
+mod tests {
+    use crate::tests_util;
+    use serde_json::json;
+
+    #[test]
+    fn schema_path() {
+        tests_util::assert_schema_path(
+            &json!({"properties": {"foo": {"$ref": "#/definitions/foo"}}, "definitions": {"foo": {"type": "string"}}}),
+            &json!({"foo": 42}),
+            "/properties/foo/type",
+        )
+    }
+}
