@@ -64,6 +64,7 @@
     clippy::redundant_closure,
     clippy::trivially_copy_pass_by_ref,
     clippy::missing_const_for_fn,
+    clippy::unseparated_literal_suffix,
     missing_debug_implementations,
     missing_docs,
     trivial_casts,
@@ -113,6 +114,7 @@ pub fn is_valid(schema: &Value, instance: &Value) -> bool {
 #[cfg(test)]
 pub(crate) mod tests_util {
     use super::JSONSchema;
+    use crate::ValidationError;
     use serde_json::Value;
 
     pub(crate) fn is_not_valid(schema: &Value, instance: &Value) {
@@ -155,6 +157,30 @@ pub(crate) mod tests_util {
             instance
         );
     }
+
+    pub(crate) fn validate(schema: &Value, instance: &Value) -> ValidationError<'static> {
+        let compiled = JSONSchema::compile(schema).unwrap();
+        let err = compiled
+            .validate(instance)
+            .expect_err("Should be an error")
+            .next()
+            .expect("Should be an error")
+            .into_owned();
+        err
+    }
+
+    pub(crate) fn assert_schema_path(schema: &Value, instance: &Value, expected: &str) {
+        let error = validate(schema, instance);
+        assert_eq!(error.schema_path.to_string(), expected)
+    }
+
+    pub(crate) fn assert_schema_paths(schema: &Value, instance: &Value, expected: &[&str]) {
+        let compiled = JSONSchema::compile(schema).unwrap();
+        let errors = compiled.validate(instance).expect_err("Should be an error");
+        for (error, schema_path) in errors.zip(expected) {
+            assert_eq!(error.schema_path.to_string(), *schema_path)
+        }
+    }
 }
 
 #[cfg(test)]
