@@ -12,8 +12,9 @@ impl JSONPointer {
         self.0
             .into_iter()
             .map(|item| match item {
-                PathChunk::Property(value) => value,
+                PathChunk::Property(value) => value.into_string(),
                 PathChunk::Index(idx) => idx.to_string(),
+                PathChunk::Keyword(keyword) => keyword.to_string(),
             })
             .collect()
     }
@@ -23,6 +24,26 @@ impl JSONPointer {
     pub fn iter(&self) -> Iter<'_, PathChunk> {
         self.0.iter()
     }
+    /// Take the last pointer chunk.
+    #[inline]
+    pub fn last(&self) -> Option<&PathChunk> {
+        self.0.last()
+    }
+
+    pub(crate) fn clone_with(&self, chunk: impl Into<PathChunk>) -> Self {
+        let mut new = self.clone();
+        new.0.push(chunk.into());
+        new
+    }
+
+    pub(crate) fn extend_with(&self, chunks: &[PathChunk]) -> Self {
+        let mut new = self.clone();
+        new.0.extend_from_slice(chunks);
+        new
+    }
+    pub(crate) fn as_slice(&self) -> &[PathChunk] {
+        &self.0
+    }
 }
 
 impl Default for JSONPointer {
@@ -37,8 +58,17 @@ impl fmt::Display for JSONPointer {
             for chunk in &self.0 {
                 f.write_char('/')?;
                 match chunk {
-                    PathChunk::Property(value) => f.write_str(value)?,
+                    PathChunk::Property(value) => {
+                        for ch in value.chars() {
+                            match ch {
+                                '/' => f.write_str("~1")?,
+                                '~' => f.write_str("~0")?,
+                                _ => f.write_char(ch)?,
+                            }
+                        }
+                    }
                     PathChunk::Index(idx) => itoa::fmt(&mut f, *idx)?,
+                    PathChunk::Keyword(keyword) => f.write_str(keyword)?,
                 }
             }
         }
@@ -64,12 +94,14 @@ impl fmt::Display for JSONPointer {
 /// The primary purpose of this enum is to avoid converting indexes to strings during validation.
 pub enum PathChunk {
     /// Property name within a JSON object.
-    Property(String),
+    Property(Box<str>),
     /// Index within a JSON array.
     Index(usize),
+    /// JSON Schema keyword.
+    Keyword(&'static str),
 }
 
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 pub(crate) struct InstancePath<'a> {
     pub(crate) chunk: Option<PathChunk>,
     pub(crate) parent: Option<&'a InstancePath<'a>>,
@@ -83,6 +115,7 @@ impl<'a> InstancePath<'a> {
         }
     }
 
+    #[inline]
     pub(crate) fn push(&'a self, chunk: impl Into<PathChunk>) -> Self {
         InstancePath {
             chunk: Some(chunk.into()),
@@ -129,7 +162,13 @@ impl<'a> IntoIterator for &'a JSONPointer {
 impl From<String> for PathChunk {
     #[inline]
     fn from(value: String) -> Self {
-        PathChunk::Property(value)
+        PathChunk::Property(value.into_boxed_str())
+    }
+}
+impl From<&'static str> for PathChunk {
+    #[inline]
+    fn from(value: &'static str) -> Self {
+        PathChunk::Keyword(value)
     }
 }
 impl From<usize> for PathChunk {
@@ -146,12 +185,19 @@ impl<'a> From<&'a InstancePath<'a>> for JSONPointer {
     }
 }
 
+impl From<InstancePath<'_>> for JSONPointer {
+    #[inline]
+    fn from(path: InstancePath<'_>) -> Self {
+        JSONPointer(path.to_vec())
+    }
+}
+
 impl From<&[&str]> for JSONPointer {
     #[inline]
     fn from(path: &[&str]) -> Self {
         JSONPointer(
             path.iter()
-                .map(|item| PathChunk::Property((*item).to_string()))
+                .map(|item| PathChunk::Property((*item).into()))
                 .collect(),
         )
     }
@@ -162,3 +208,18 @@ impl From<&[PathChunk]> for JSONPointer {
         JSONPointer(path.to_vec())
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use super::JSONPointer;
+    use serde_json::json;
+
+    #[test]
+    fn json_pointer_to_string() {
+        let chunks = ["/", "~"];
+        let pointer = JSONPointer::from(&chunks[..]).to_string();
+        assert_eq!(pointer, "/~1/~0");
+        let data = json!({"/": {"~": 42}});
+        assert_eq!(data.pointer(&pointer), Some(&json!(42)))
+    }
+}
